# z2html.sed - Z2HTML: simple Fuzz/LaTeX to HTML translator for Z notation
#
# Use GIFs (not symbol font) for Z symbols, HTML tables for Z boxes
# Use *-m.gif, looks best with medium text size
# Set each Z paragraph inside <div class="zed"> ... </div> for stylesheet
# Set in-line Z text (LaTeX $...$) inside <var class="zed"> ... </var> ditto
#
# 11-Jul-1999  J. Jacky  First attempts, used symbol font for most Z characters
#  9-Oct-1999  J. Jacky  Replace all font face=symbol... with GIF's
# 23-Nov-1999  J. Jacky  Begin stylesheet version
#  7-Jan-2000  J. Jacky  Replace <p ...> around tables with <div ...>
# 13-Jan-2000  J. Jacky  Rename zgif directory to zimg (maybe use .png later)
# 15-Jan-2000  J. Jacky  Add closing </tr> in tabular, zed paragraphs
#                        Don't put &nbsp before <em> etc. in informal paragrphs
# Early-2000  I. Toyn  Lots of changes to suit CADiZ documentation.

# Header

1i\
<html>\
\
<head>\
<link rel="stylesheet" href="bin/zed.css" type="text/css">\

# Footer

$a\
\
<\/body>\
<\/html>

# LaTex verbatim

/\\begin{verbatim}/,/\\end{verbatim}/{
s/\\begin{verbatim}/<pre>/
s/\\end{verbatim}/<\/pre>/
b
}

/\\begin{quiet}/,/\\end{quiet}/d

# Get rid of Fuzz comment lines, otherwise Z behind them will appear in HTML

/^%%/d

# Get rid of % LaTeX comments but not \% legitimate LaTeX percent
/^%.*$/d
s/\([^\\]\)%.*$/\1/
s/\\%/%/g

# Must convert these before writing out *any* HTML tags!
# But can't use any & before handling LaTeX tabular and argue

s/\\>/\\latex-tab/g
s/</\\html-amp-lt;/g
s/>/\\html-amp-gt;/g

s/\\verb!\([^!]*\)!/<tt>\1<\/tt>/g

s/\\chapter{\([^}]*\)}.*$/<TITLE>CADiZ: \1<\/TITLE>\
<\/head>\
<body>\
<A HREF=".\/home.html"><IMG SRC="cadizwide.gif" ALT="CADiZ"><\/A>/

s/\\begin{pagepath}/<h1>/
s/\\parent{\([^}]*\).pdf}{\([^}]*\)}/<a href="\1.html">\2<\/a> \/ /g
s/\\self{\([^}]*\)}/\1<\/h1>/
s/\\end{pagepath}/\
<hr>/

s/\\footstamp{\([^}]*\)}/<hr>\
<address>\1<\/address>/

s/\\imgsrc{\([^}]*\)}{\([^}]*\)}/<img src=\1.gif \2>/

s/\\index{[^}]*}//g

s/\\CADiZ/CADi<img height=11 width=10 alt="!num" src="zimg\/num-m.gif">/g
s/\\LaTeX/L<sup><font size=-2>A<\/font><\/sup>T<sub>E<\/sub>X/g

# LaTex tabbing - must convert before inserting any other tags
#
# tabbing, tabular, argue environments are class="mixed"
# because they can contain both Z and informal text

/\\begin{tabbing}/,/\\end{tabbing}/ {
	s/\\begin{tabbing}/<div class="mixed">\
<table border=0 cellpadding=0 cellspacing=0>/g
	s/\\end{tabbing}/<\/table>\
<\/div>\
/g
	/^[ 	]*</!s/.*/<tr> <td>&<\/td><\/tr>/
	s/\\=/<\/td> <td>/g
	s/\\latex-tab/<\/td> <td>/g
}

# LaTex tabular 

/\\begin{tabular}/,/\\end{tabular}/ {
	s/\\begin{tabular}.*$/<div class="mixed">\
<table border=0 cellpadding=0 cellspacing=0>/g
	s/\\end{tabular}/<\/table>\
<\/div>\
/g
	/^[ 	]*</!s/.*/<tr> <td>&<\/td><\/tr>/
	s/\&/<\/td> <td>/g
}

# LaTeX argue environment (proofs etc.)

/\\begin{argue}/,/\\end{argue}/ {
	s/\\begin{argue}/<div class="mixed">\
<table border=0 cellpadding=0 cellspacing=0>/g
	s/\\end{argue}/<\/table>\
<\/div>\
/g
	/^[ 	]*</!s/^\([^&]*\)\&\(.*\)$/<tr> <td>\$\1\$<\/td><td align=right>\2<\/td><\/tr>/g
}

# LaTeX (oz) infrule environment

s/\\begin{infrule}/<div class="mixed"><center><table border=0 cellpadding=0 cellspacing=0><tr><td>/
s/\\derive/<HR>/
s/\\end{infrule}/<\/td><\/tr><\/table><\/center><\/div>/

# LaTex center

s/\\begin{center}/<center>/
s/\\end{center}/<\/center>/

# LaTex itemize

s/\\begin{description}/<dl>/
s/\\begin{itemize}/<ul>/
s/\\begin{enumerate}/<ol>/
s/\\item\[\(.*\)\]/<dt>\1<\/dt>/g
s/\\item/<li>/g
s/\\end{enumerate}/<\/ol>/
s/\\end{itemize}/<\/ul>/
s/\\end{description}/<\/dl>/

# Now we've translated all the LaTeX & so...

s/\\html-amp-/\&/g

# In-line Z text

#s/\$\([^$]*\)\$/<var class="zed">\1<\/var>/g This line fails to exclude \$
s/\$\$/<var class="zed"><\/var>/g
s/^\$\([^$]*\)\$/<var class="zed">\1<\/var>/g
s/\([^\\]\)\$\([^$]*[^\\]\)\$/\1<var class="zed">\2<\/var>/g

# Single line math paragraph

s/\\\[\(.*\)\\\]/<div class="zed">\
<table border=0 cellpadding=0 cellspacing=0>\
<tr> <td class="zed">\1<\/td><\/tr>\
<\/table>\
<\/div>\
/g

# Section headers

s/\\begin{zsection}/\\begin{zed}/
s/\\end{zsection}/\\end{zed}/

# CADiZ conjectures

s/\\begin{theorem}{[ 	]*}/\\begin{zed}/
s/\\begin{theorem}{\([^}]*\)}/\\begin{zed}\
\1 ==/
s/\\end{theorem}/\\end{zed}/

# Single line zed paragraph

s/\\begin{zed}\(.*\)\\end{zed}/<div class="zed">\
<table border=0 cellpadding=0 cellspacing=0>\
<tr> <td class="zed">\1<\/td><\/tr>\
<\/table>\
<\/div>\
/g

# Multiline zed paragraph

/\\begin{zed}/,/\\end{zed}/ {
	s/\\begin{zed}/<div class="zed">\
<table border=0 cellpadding=0 cellspacing=0>/g
	s/\\end{zed}/<\/table>\
<\/div>\
/g
	/^[ 	]*</!s/.*/<tr> <td class="zed">&<\/td><\/tr>/
	s/\\also/<img width=1 height=10 align=bottom alt=" " src="zimg\/clear.gif">/
}


# Single line axdef box

# We use ! not LaTeX \ inside alt="..." to avoid confusion

s/\\begin{axdef}\(.*\)\\end{axdef}/<div class="zed">\
<table border=0 cellpadding=0 cellspacing=0>\
<tr> <td width=1 rowspan=99><br><\/td><\/tr>\
<tr> <td width=10><\/td><td class="zed">\1<\/td><\/tr>\
<tr><td><img width=1 height=1 alt="!end{axdef}" src="zimg\/clear.gif"><\/td><\/tr>\
<\/table>\
<\/div>\
/g

# Axiomatic definition box

/\\begin{axdef}/,/\\end{axdef}/ {
	s/\\begin{axdef}/<div class="zed">\
<table border=0 cellpadding=0 cellspacing=0>\
<tr> <td width=1 rowspan=99 bgcolor=black><br><\/td>\
     <td width=10><\/td><\/tr>/
	s/\\where/<tr> <td height=10><\/td><\/tr>\
<tr> <td width=10 height=1 bgcolor=black><img width=1 height=1 alt="!where" src="zimg\/clear.gif"><\/td>\
      <td height=1 bgcolor=black><img width=1 height=1 alt=" " src="zimg\/clear.gif"><\/td>\
      <td height=1><img width=75 height=1 alt=" " src="zimg\/black-75.gif"><\/td><\/tr>\
<tr> <td height=10><\/td><\/tr>/
	s/\\end{axdef}/<\/table>\
<\/div>\
/
	/^[ 	]*</!s/.*/<tr> <td><\/td><td class="zed" colspan=2>&<\/td><\/tr>/
	s/\\also/<img width=1 height=10 align=bottom alt=" " src="zimg\/clear.gif">/
}

# Generic schema definition box

/\\begin{schema}{.*}\[.*\]/,/\\end{schema}/ {
	s/\\begin{schema}{\(.*\)}\[\(.*\)\]/<div class="zed">\
<table border=0 cellpadding=0 cellspacing=0>\
<tr> <td><\/td>\
     <td><\/td>\
     <td class="zed">\1 \[\2\]<\/td><\/tr>\
<tr> <td width=1 rowspan=99 bgcolor=black><br><\/td><\/tr>\
<tr> <td width=10 height=1 bgcolor=black><img width=10 height=1 alt=" " src="zimg\/clear.gif"><\/td>\
     <td height=1><\/td>\
     <td width=200 height=1 bgcolor=black><img width=200 height=1 alt=" " src="zimg\/clear.gif"><\/td>\
     <td width=1 height=1 bgcolor=black><img width=1 height=1 alt=" " src="zimg\/clear.gif"><\/td><\/tr>\
<tr> <td><\/td><td><\/td><td><\/td><td><\/td><td width=1 height=3 bgcolor=black><img width=1 height=1 alt=" " src="zimg\/clear.gif"><\/td><\/tr>\
<tr> <td width=10 height=1 bgcolor=black><img width=10 height=1 alt=" " src="zimg\/clear.gif"><\/td>\
     <td height=1><\/td>\
     <td width=200 height=1 bgcolor=black><img width=200 height=1 alt=" " src="zimg\/clear.gif"><\/td>\
     <td width=1 height=1 bgcolor=black><img width=1 height=1 alt=" " src="zimg\/clear.gif"><\/td><\/tr>\
<tr> <td><\/td><td><\/td><td><\/td><td><\/td><td width=1 height=10 bgcolor=black><img width=1 height=1 alt=" " src="zimg\/clear.gif"><\/td><\/tr>/
	s/\\where/<tr> <td height=10><\/td><\/tr>\
<tr> <td height=1 bgcolor=black><img width=1 height=1 alt="!where" src="zimg\/clear.gif"><\/td>\
      <td height=1 bgcolor=black><img width=1 height=1 alt="!where" src="zimg\/clear.gif"><\/td>\
      <td height=1><img width=75 height=1 alt="!where" src="zimg\/black-75.gif"><\/td><\/tr>\
<tr> <td height=10><\/td><\/tr>/
	s/\\end{schema}/<tr> <td><\/td><td><\/td><td><\/td><td><\/td><td width=1 height=10 bgcolor=black><img width=1 height=1 alt=" " src="zimg\/clear.gif"><\/td><\/tr>\
<tr> <td colspan=4 height=1 bgcolor=black><img width=300 height=1 alt="!end{schema}" src="zimg\/clear.gif"><\/td><\/tr>\
<\/table>\
<\/div>\
/
	/^[ 	]*</!s/.*/<tr> <td><\/td><td class="zed" colspan=2>&<\/td><\/tr>/
	s/\\also/<img width=1 height=10 align=bottom alt=" " src="zimg\/clear.gif">/
}

# Schema definition box
#
# Table has columns:
# 1 vertical line
# 2 gap
# 3 schema name
# 4 rest of text
# 5 encapsulation hooks
# Text inside box (both decl and pred) spans two columns
# Schema name sits by itelf on a row above the box top borders
# netscape seems to be behaving a bit strangely re cols 1 and 2.

/\\begin{schema}{.*}/,/\\end{schema}/ {
	s/\\begin{schema}{\(.*\)}/<div class="zed">\
<table border=0 cellpadding=0 cellspacing=0>\
<tr> <td width=1><\/td>\
     <td width=10><\/td>\
     <td class="zed">\1<\/td><\/tr>\
<tr> <td width=1 rowspan=99 bgcolor=black><br><\/td><\/tr>\
<tr> <td width=10 height=1 bgcolor=black><img width=1 height=1 alt=" " src="zimg\/clear.gif"><\/td>\
     <td><\/td>\
     <td width=200 height=1 bgcolor=black><img width=200 height=1 src="zimg\/clear.gif"><\/td>\
     <td width=1 height=1 bgcolor=black><img width=1 height=1 alt=" " src="zimg\/clear.gif"><\/td><\/tr>\
<tr> <td><\/td><td><\/td><td><\/td><td><\/td><td width=1 height=10 bgcolor=black><img width=1 height=1 alt=" " src="zimg\/clear.gif"><\/td><\/tr>/
	s/\\where/<tr> <td height=10><\/td><\/tr>\
<tr> <td height=1 bgcolor=black><img width=1 height=1 alt="!where" src="zimg\/clear.gif"><\/td>\
     <td height=1 bgcolor=black><img width=1 height=1 alt="!where" src="zimg\/clear.gif"><\/td>\
      <td height=1><img width=75 height=1 alt="!where" src="zimg\/black-75.gif"><\/td><\/tr>\
<tr> <td height=10><\/td><\/tr>/
	s/\\end{schema}/<tr> <td><\/td><td><\/td><td><\/td><td><\/td><td width=1 height=10 bgcolor=black><img width=1 height=1 alt=" " src="zimg\/clear.gif"><\/td><\/tr>\
<tr> <td colspan=5 height=1 bgcolor=black><img width=1 height=1 alt="!end{schema}" src="zimg\/clear.gif"><\/td><\/tr>\
<\/table>\
<\/div>\
/
	/^[ 	]*</!s/.*/<tr> <td><\/td><td class="zed" colspan=2>&<\/td><\/tr>/
	s/\\also/<img width=1 height=10 align=bottom alt=" " src="zimg\/clear.gif">/
}

# Generic axiomatic description box

/\\begin{gendef}\[.*\]/,/\\end{gendef}/ {
	s/\\begin{gendef}\[\(.*\)\]/<div class="zed">\
<table border=0 cellpadding=0 cellspacing=0>\
<tr> <td><\/td>\
     <td><\/td>\
     <td class="zed">\[\1\]<\/td><\/tr>\
<tr> <td width=1 rowspan=99 bgcolor=black><br><\/td><\/tr>\
<tr> <td width=10 height=1 bgcolor=black><img width=10 height=1 alt=" " src="zimg\/clear.gif"><\/td>\
     <td height=1><\/td>\
     <td width=200 height=1 bgcolor=black><img width=200 height=1 alt=" " src="zimg\/clear.gif"><\/td><\/tr>\
<tr> <td height=3><\/td><\/tr>\
<tr> <td width=10 height=1 bgcolor=black><img width=10 height=1 alt=" " src="zimg\/clear.gif"><\/td>\
     <td height=1><\/td>\
     <td width=200 height=1 bgcolor=black><img width=200 height=1 alt=" " src="zimg\/clear.gif"><\/td><\/tr>\
<tr> <td height=10><\/td><\/tr>/
	s/\\where/<tr> <td height=10><\/td><\/tr>\
<tr> <td height=1 bgcolor=black><img width=1 height=1 alt="!where" src="zimg\/clear.gif"><\/td>\
      <td height=1 bgcolor=black><img width=1 height=1 alt="!where" src="zimg\/clear.gif"><\/td>\
      <td height=1><img width=75 height=1 alt="!where" src="zimg\/black-75.gif"><\/td><\/tr>\
<tr> <td height=10><\/td><\/tr>/
	s/\\end{gendef}/<tr> <td height=10><\/td><\/tr>\
<\/table>\
<\/div>\
/
	/^[ 	]*</!s/.*/<tr> <td><\/td><td class="zed" colspan=2>&<\/td><\/tr>/
	s/\\also/<img width=1 height=10 align=bottom alt=" " src="zimg\/clear.gif">/
}

# White space: line break, tab, space, indents

s/\\\\//g
s/	//g
s/~/\&nbsp;/g
s/\\ /\&nbsp;/g
s/\\t\([1-9]\)/<img width=\10 height=1 align=bottom alt="    " src="zimg\/clear.gif">/


# Logic

# We use ! not LaTeX \ inside alt="..." to avoid confusion
# between LaTeX commands we want to replace and literal text we want in alt=
# Replace all ! with \ at end of the script, after all LaTeX cmds processed

s/\\lnot/<img width=7 height=7 alt="!lnot" src="zimg\/lnot-m.gif">/g
s/\\land/<img width=8 height=7 alt="!land" src="zimg\/land-m.gif">/g
s/\\lor/<img width=8 height=7 alt="!lor" src="zimg\/lor-m.gif">/g
s/\\xor/<img width=8 height=9 alt="!xor" src="zimg\/xor-m.gif">/g
s/\\implies/<img width=13 height=7 alt="!implies" src="zimg\/implies-m.gif">/g
s/\\iff/<img width=13 height=7 alt="!iff" src="zimg\/iff-m.gif">/g
s/\\forall/<img width=9 height=10 alt="!forall" src="zimg\/forall-m.gif">/g
s/@\([^c]\)/<img width=7 height=7 alt="@" src="zimg\/spot-m.gif">\1/g
s/\\spot/<img width=7 height=7 alt="@" src="zimg\/spot-m.gif">/g
s/\\endpat/<img width=7 height=7 alt="@" src="zimg\/spot-m.gif">/g
s/\\exists/<img width=6 height=10 alt="!exists" src="zimg\/exists-m.gif">/g
s/\\vdash/<img width=8 height=11 alt="!vdash" src="zimg\/vdash-m.gif">/g
s/\\thrm/\\vdash?/g
s/\\models/<img width=8 height=11 alt="!models" src="zimg\/models-m.gif">/g
s/\\vdash/<img width=8 height=11 alt="!vdash" src="zimg\/vdash-m.gif">/g
s/\\dagger/<img width=6 height=11 alt="!dagger" src="zimg\/dagger-m.gif">/g
s/\\LET/let/g
s/\\zlet/let/g

# LaTeX commands that begin with \in... must precede \in itself, ditto \cap

s/\\inbag/<img width=7 height=7 alt="!inbag" src="zimg\/inbag-m.gif">/g
s/\\infix/infix/g
s/\\input/input/g
s/\\inrel{\([^}]*\)}/<u>\1<\/u>/g
s/\\inj/<img  width=14 height=8 alt="!inj" src="zimg\/inj-m.gif">/g
s/\\inv/<sup>~<\/sup>/g

s/\\caption/caption/g

# Sets and expressions

s/\\neq/<img width=7 height=7 alt="!neq" src="zimg\/neq-m.gif">/g
s/\\in/<img width=7 height=7 alt="!in" src="zimg\/in-m.gif">/g
s/\\notin/<img width=7 height=11 align=baseline alt="!notin" src="zimg\/notin-m.gif">/g
s/\\emptyset/<img width=11 height=11 alt="!emptyset" src="zimg\/emptyset-m.gif">/g
s/\\subseteq/<img width=9 height=10 align=baseline alt="!subseteq" src="zimg\/subseteq-m.gif">/g
s/\\subset/<img width=9 height=7 alt="!subset" src="zimg\/subset-m.gif">/g
s/\\{/{/g
s/\\}/}/g
s/\\lambda/<img width=8 height=10 alt="!lambda" src="zimg\/lambda-m.gif">/g
s/\\mu/<img width=7 height=10 alt="!mu" align=baseline src="zimg\/mu-m.gif">/g
s/\\IF/if/g
s/\\THEN/then/g
s/\\ELSE/else/g
s/\\cross/<img width=7 height=7 alt="!cross" src="zimg\/cross-m.gif">/g
s/\\power/<img height=11 width=9 alt="!power" src="zimg\/power-m.gif">/g
s/\\finset/<img height=11 width=8 alt="!finset" src="zimg\/finset-m.gif">/g
s/\\cap/<img width=10 height=7 alt="!cap" src="zimg\/cap-m.gif">/g
s/\\cup/<img width=10 height=7 alt="!cup" src="zimg\/cup-m.gif">/g
s/\\bigcap/<img width=11 height=14 alt="!bigcap" align="baseline" src="zimg\/bigcap-m.gif">/g
s/\\bigcup/<img width=11 height=14 alt="!bigcup" align="baseline" src="zimg\/bigcup-m.gif">/g
s/\\setminus/\\/g
s/\\symdiff/<img width=11 height=11 alt="!symdiff" align="baseline" src="zimg\/symdiff-m.gif">/g
s/\\listarg/,,/g
s/\\varg/\\_/g

# Free types

s/\\ldata/<img width=8 height=14 align=baseline alt="!ldata" src="zimg\/ldata-m.gif">/g
s/\\rdata/<img width=8 height=14 align=baseline alt="!rdata" src="zimg\/rdata-m.gif">/g

# LaTeX commands that begin with \ran... must precede \ran itself

s/\\rangle/<img width=5 height=14 align=baseline alt="!rangle" src="zimg\/rangle-m.gif">/g
s/\\function/function/g
s/\\relation/relation/g
s/\\generic/generic/g
s/\\leftassoc/leftassoc/g
s/\\rightassoc/rightassoc/g
s/\\SECTION/section/g
s/\\parents/parents/g

# Relations, except \inrel and \inv must precede \in above

s/\\rel/<img width=13 height=8 alt="!rel" src="zimg\/rel-m.gif">/g
s/\\mapsto/<img height=9 width=10 alt="!mapsto" src="zimg\/mapsto-m.gif">/g
s/\\dom/dom/g
s/\\ran/ran/g
s/\\id/id/g
s/\\comp/<img width=6 height=12 alt="!comp" src="zimg\/comp-m.gif" align=baseline>/g
s/\\circ/<img width=7 height=7 alt="!circ" src="zimg\/circ-m.gif">/g
s/\\dres/<img height=11 width=10 alt="!dres" src="zimg\/dres-m.gif">/g
s/\\ndres/<img height=11 width=10  alt="!ndres" src="zimg\/ndres-m.gif">/g
s/\\rres/<img height=11 width=10 alt="!rres" src="zimg\/rres-m.gif">/g
s/\\nrres/<img height=11 width=10 alt="!nrres" src="zimg\/nrres-m.gif">/g
s/\\limg/<img height=14 width=7 alt="!limg" src="zimg\/limg-m.gif" align=baseline>/g
s/\\rimg/<img height=14 width=7 alt="!rimg" src="zimg\/rimg-m.gif" align=baseline>/g
s/\\plus/<sup>+<\/sup>/g
s/\\star/<sup>*<\/sup>/g
s/\\oplus/<img width=9 height=9 alt="!oplus" src="zimg\/oplus-m.gif">/g

# Functions, except \inj must precede \in above

s/\\pfun/<img width=14 height=8 alt="!pfun" src="zimg\/pfun-m.gif">/g
s/\\fun/<img width=14 height=8 alt="!fun" src="zimg\/fun-m.gif">/g
s/\\pinj/<img width=14 height=8 alt="!pinj" src="zimg\/pinj-m.gif">/g
s/\\bij/<img width=14 height=8 alt="!bij" src="zimg\/bij-m.gif">/g
s/\\psurj/<img width=14 height=8 alt="!psurj" src="zimg\/psurj-m.gif">/g
s/\\surj/<img width=14 height=8 alt="!surj" src="zimg\/surj-m.gif">/g
s/\\ffun/<img width=14 height=8 alt="!ffun" src="zimg\/ffun-m.gif">/g
s/\\finj/<img width=14 height=8 alt="!finj" src="zimg\/finj-m.gif">/g

# Numbers and arithmetic

s/\\#/#/g
s/\([^\\]\)_\([^{]\)/\1<sub><font size=-3>\2<\/font><\/sub>/g
s/\([^\\]\)^\([^{<]\)/\1<sup><font size=-3>\2<\/font><\/sup>/g
s/\([^\\]\)_{\([^}]*\)}/\1<sub><font size=-3>\2<\/font><\/sub>/g
s/\([^\\]\)^{\([^}]*\)}/\1<sup><font size=-3>\2<\/font><\/sup>/g
#s/_1/<sub><font size=-3>1<\/font><\/sub>/g
#s/_2/<sub><font size=-3>2<\/font><\/sub>/g
#s/_3/<sub><font size=-3>3<\/font><\/sub>/g
s/\\arithmos/<img height=11 width=9 alt="!arithmos" src="zimg\/arithmos-m.gif">/g
s/\\num/<img height=11 width=10 alt="!num" src="zimg\/num-m.gif">/g
s/\\nat/<img height=11 width=10 alt="!nat" src="zimg\/nat-m.gif">/g
s/\\fraction/\\rat<sub>+<\/sub>/g
s/\\rat/<img height=11 width=8 alt="!rat" src="zimg\/rat-m.gif">/g
s/\\real/<img height=11 width=9 alt="!real" src="zimg\/real-m.gif">/g
s/\\divides/<img width=9 height=7 alt="!divides" src="zimg\/divides-m.gif">/g
s/\\div/div/g
s/\\mod/mod/g
s/\\leq/<img width=7 height=9 alt="!leq" src="zimg\/leq-m.gif">/g
s/\\geq/<img width=7 height=9 alt="!geq" src="zimg\/geq-m.gif">/g
s/\\upto/\.\./g
s/\\negate/-/g

# Sequences, except \rangle must precede \ran above

s/\\seq/seq/g
s/\\iseq/iseq/g
s/\\langle/<img width=5 height=14 align=baseline alt="!langle" src="zimg\/langle-m.gif">/g
s/\\cat/<img width=12 height=11 alt="!cat" src="zimg\/cat-m.gif">/g
s/\\dsum/+\//g
s/\\dcat/<img width=17 height=14 align=baseline alt="!dcat" src="zimg\/dcat-m.gif">/g
s/\\prefix/prefix/g
s/\\suffix/suffix/g
s/\\partition/partition/g
s/\\disjoint/disjoint/g
s/\\extract/<img width=5 height=14 align=baseline alt="!extract" src="zimg\/extract-m.gif">/g
s/\\filter/<img width=5 height=14 align=baseline alt="!filter" src="zimg\/filter-m.gif">/g
s/\\nearrow/<img width=11 height=11 align=baseline alt="!nearrow" src="zimg\/nearrow-m.gif">/g
s/\\searrow/<img width=11 height=11 align=baseline alt="!searrow" src="zimg\/searrow-m.gif">/g
s/\\swarrow/<img width=11 height=11 align=baseline alt="!swarrow" src="zimg\/swarrow-m.gif">/g
s/\\nwarrow/<img width=11 height=11 align=baseline alt="!nwarrow" src="zimg\/nwarrow-m.gif">/g

# Bags - except \inbag must precede \in, above

s/\\bag/bag/g
s/\\lbag/<img width=7 height=14 align=baseline alt="!lbag" src="zimg\/lbag-m.gif">/g
s/\\rbag/<img width=7 height=14 align=baseline alt="!rbag" src="zimg\/rbag-m.gif">/g
s/\\bcount/<img width=5 height=14 align=baseline alt="!bcount" src="zimg\/bcount-m.gif">/g
s/\\otimes/<img width=9 height=9 alt="!otimes" src="zimg\/otimes-m.gif">/g
s/\\subbageq/<img width=9 height=10 align=baseline alt="!subbageq" src="zimg\/subbageq-m.gif">/g
s/\\uplus/<img width=11 height=7 alt="!uplus" src="zimg\/uplus-m.gif">/g
s/\\uminus/<img width=11 height=7 alt="!uminus" src="zimg\/uminus-m.gif">/g

# Schema calculus

s/\\defs/<img height=11 width=11 alt="!defs" src="zimg\/defs-m.gif">/g
s/\\sdef/<img height=11 width=11 alt="!defs" src="zimg\/defs-m.gif">/g
s/\\theta/<img width=5 height=10 alt="!theta" src="zimg\/theta-m.gif">/g
s/\\semi/<img width=6 height=12 alt="!semi" align=baseline src="zimg\/comp-m.gif">/g
s/\\project/<img width=5 height=14 align=baseline alt="!project" src="zimg\/filter-m.gif">/g
s/\\pipe/\&gt;\&gt;/g
s/\\hide/\\/g
s/\\pre/pre/g
s/\\lblot/<img width=5 height=14 align=baseline alt="!lblot" src="zimg\/lblot-m.gif">/g
s/\\rblot/<img width=5 height=14 align=baseline alt="!rblot" src="zimg\/rblot-m.gif">/g
s/\\zovr/<img width=11 height=11 alt="!zovr" align=baseline src="zimg\/zovr-m.gif">/g

# Conventions

s/\\Delta/<img width=7 height=10 alt="!Delta" src="zimg\/Delta-m.gif">/g
s/\\Xi/<img width=9 height=10 alt="!Xi" src="zimg\/Xi-m.gif">/g

# Z standard metanotation

s/\\trarrow/\&nbsp;\&nbsp;<img width=18 height=7 alt="!trarrow" src="zimg\/trarrow-m.gif">\&nbsp;\&nbsp;/g
s/\\tau/<img width=5 height=7 alt="!tau" src="zimg\/tau-m.gif">/g
#s/\\tau/\&tau;/g
s/\\comment{\(.*\)}/<em>\1<\/em>/g
s/\\chartuple/chartuple/g
s/\\undecor/undecor/g
s/\\equiv/<img width=7 height=7 alt="!equiv" src="zimg\/equiv-m.gif">/g
s/\\leftarrow/<img width=14 height=8 alt="!leftarrow" src="zimg\/leftarrow-m.gif">/g

# Refinement

s/\\specstmt/<img width=7 height=10 alt="!specstmt" src="zimg\/Delta-m.gif">/g

# LaTeX stuff

# Translate these first because they can occur inside section headings etc.

s/{\\em \([^}]*\)}/<em>\1<\/em>/g
s/{\\bf \([^}]*\)}/<b>\1<\/b>/g
s/{\\tt \([^}]*\)}/<tt>\1<\/tt>/g
s/\\mbox{\([^}]*\)}/\1/g

# Citations
s/\\cite{\([^}]*\)}/\[\1\]/g

# Labels often occur on same line as section headings so translate them first
s/\\\([^\\]*\)\\label{\(.*\)}/<a name="\2">\\\1<\/a>/
s/\\\([^\\]*\)\\hypertarget{\(.*\)}{}/<a name="\2">\\\1<\/a>/

# For now just get rid of labels
#s/\\label{[^}]*}//g

# Cross-ref links.
# s/\\beginref \([^\\]*\)\\finishref{\([^}]*\)}/<a href="\2">\1<\/a>/g
s/\\beginref \(.*\)\\ftpref{\([^}]*\)}/<a href="\2">\1<\/a>/g
s/\\href{\.\/\([^}]*\).pdf}{\([^}]*\)}/<a href=".\/\1.html">\2<\/a>/g
s/\\href{\([^}]*\)}{\([^}]*\)}/<a href="\1">\2<\/a>/g
s/\\hyperlink{\.\/\([^}]*\).pdf}{\([^}]*\)}/<a href=".\/\1.html">\2<\/a>/g
s/\\hyperlink{\([^}]*\)}{\([^}]*\)}/<a href="#\1">\2<\/a>/g
s/\\ref{\(.*\)}\(.*\)/<a href="\#\1">\2<\/a>/g

s/\\webcounter{\([^}]*\)}/<p><img src=\1><\/p>/

s/\\chapter{\([^}]*\)}/<h1>\1<\/h1>/g
#s/\\section{\([^}]*\)}/<h2><a name="\1">\1<\/a><\/h2>/g
#s/\\section\*{\([^}]*\)}/<h2><a name="\1">\1<\/a><\/h2>/g
#s/\\subsection{\([^}]*\)}/<h3><a name="\1">\1<\/a><\/h3>/g
#s/\\subsection\*{\([^}]*\)}/<h3><a name="\1">\1<\/a><\/h3>/g
#s/\\subsubsection{\([^}]*\)}/<h4><a name="\1">\1<\/a><\/h4>/g
#s/\\subsubsection\*{\([^}]*\)}/<h4><a name="\1">\1<\/a><\/h4>/g
s/\\section{\([^}]*\)}/<h2>\1<\/h2>/g
s/\\section\*{\([^}]*\)}/<h2>\1<\/h2>/g
s/\\subsection{\([^}]*\)}/<h3>\1<\/h3>/g
s/\\subsection\*{\([^}]*\)}/<h3>\1<\/h3>/g
s/\\subsubsection{\([^}]*\)}/<h4>\1<\/h4>/g
s/\\subsubsection\*{\([^}]*\)}/<h4>\1<\/h4>/g
s/\\footnote{\([^}]*\)}/ (\1)/g

# Put paragraph headings on a line by themselves so text is clearly informal
s/\\paragraph{\([^}]*\)}/<h4>\1<\/h4>\
\
/g

# CADiZ tactics -- not earlier because of \map & \mapsto, \par & \paragraph

s/\\begin{tactic}/\\begin{zed}/g
s/\\end{tactic}/\\end{zed}/g
s/\\patgoal/pat<sub><font size=-3>goal<\/font><\/sub>/g
s/\\patante/pat<sub><font size=-3>ante<\/font><\/sub>/g
s/\\patcons/pat<sub><font size=-3>cons<\/font><\/sub>/g
s/\\patpred/pat<sub><font size=-3>pred<\/font><\/sub>/g
s/\\patexpr/pat<sub><font size=-3>expr<\/font><\/sub>/g
s/\\match/match/g
s/\\as/as/g
s/\\rec/rec/g
s/\\map/map/g
s/\\exprs/exprs/g
s/\\expr/expr/g
s/\\pred/pred/g
s/\\declsafter/declsafter/g
s/\\declsbefore/declsbefore/g
s/\\decls/decls/g
s/\\declof/declof/g
s/\\decl/decl/g
s/\\decnof/decnof/g
s/\\decn/decn/g
s/\\names/names/g
s/\\name/name/g
s/\\string/string/g
s/\\stxt/stxt/g
s/\\goal/goal/g
s/\\wexprs/\\_exprs/g
s/\\wexpr/\\_expr/g
s/\\wpred/\\_pred/g
s/\\wdecls/\\_decls/g
s/\\wdecl/\\_decl/g
s/\\wnames/\\_names/g
s/\\wname/\\_name/g
s/\\wstxt/\\_stxt/g
s/\\wtype/\\_type/g
s/\\parsedecls/parse<sub><font size=-3>decls<\/font><\/sub>/g
s/\\parsedecl/parse<sub><font size=-3>decl<\/font><\/sub>/g
s/\\parseexprs/parse<sub><font size=-3>exprs<\/font><\/sub>/g
s/\\parseexpr/parse<sub><font size=-3>expr<\/font><\/sub>/g
s/\\parsename/parse<sub><font size=-3>name<\/font><\/sub>/g
s/\\parsepred/parse<sub><font size=-3>pred<\/font><\/sub>/g
s/\\parsestxt/parse<sub><font size=-3>stxt<\/font><\/sub>/g
s/\\antecedent/antecedent/g
s/\\consequent/consequent/g
s/\\typeof/typeof/g
s/\\type/type/g
s/\\par/||/g
s/\\strapp/++/g

s/\\begin{quote}/<blockquote>/g
s/\\end{quote}/<\/blockquote>/g

# Just get rid of untranslated LaTeX environment directives for now
/^\\[^ 	]*{[^}]*}$/d
s/\\[^ 	]*{[^}]*}//g

s/\\footnote{\([^}]*\)$/ (\1/g
s/^\([^{}]*\)}/\1)/g

# s/\\.*{.*}//g -- This test is too weak, deletes legitimate Fuzz/LaTeX also

# Misc. LaTeX commands 
# Commands not explicitly handled in this script will appear verbatim in HTML

s/\\dots/.../g
s/\\ldots/.../g
s/\\_/_/g
s/\\\$/\$/g
s/\\backslash[ 	]*/\\/g
s/\\textasciitilde/~/g
s/\\\&/\&/g
s/---/ - /g
s/{\\scriptsize//g
# \\notnewpar translates to something that deceives paragraph.awk
s/\\notnewpar/<div class="dummy">\
<\/div>/

# Get rid of \kill in \tabbing but keep the \kill lines
s/\\kill//g

# Get rid of these
/\\newpage/d

# Finally, replace ! with LaTeX prefix char \ inside alt="..."
s/alt="!/alt="\\/g

# Have a nice day.
